@@ -14,13 +14,14 @@ use nfqueue;
 
 use NetPacket::IP qw(IP_PROTO_TCP);
 use NetPacket::TCP;
+use Socket qw(AF_INET AF_INET6);
 
 my $q;
 
 sub cleanup()
 {
 	print "unbind\n";
-	$q->unbind();
+	$q->unbind(AF_INET);
 	print "close\n";
 	$q->close();
 }
@@ -56,7 +57,7 @@ $q = new nfqueue::queue();
 print "open\n";
 $q->open();
 print "bind\n";
-$q->bind();
+$q->bind(AF_INET);
 
 $SIG{INT} = "cleanup";
 
@@ -69,6 +70,8 @@ $q->set_callback(\&cb);
 print "creating queue\n";
 $q->create_queue(0);
 
+$q->set_queue_maxlen(5000);
+
 print "trying to run\n";
 $q->try_run();
 
