@@ -38,7 +38,7 @@ print "open"
 q.open()
 
 print "bind"
-q.bind();
+q.bind(AF_INET);
 
 #print "setting callback (should fail, wrong arg type)"
 #try:
@@ -52,6 +52,8 @@ q.set_callback(cb)
 print "creating queue"
 q.create_queue(0)
 
+q.set_queue_maxlen(5000)
+
 print "trying to run"
 try:
 	q.try_run()
@@ -60,7 +62,7 @@ except KeyboardInterrupt, e:
 
 
 print "unbind"
-q.unbind()
+q.unbind(AF_INET)
 
 print "close"
 q.close()
